# Getting Started

The artifact consists of two main components:

- GDBMeter, the tool developed as part of this paper, utilizes the Predicate Partitioning approach to uncover all bugs reported in this study.
- A SQLite database of the bugs that were reported as part of this paper including additional meta information.

# GDBMeter

GDBMeter is a [Java](https://www.java.com/) application built using Gradle. Since we believe Java and Gradle to be widely-used software, and since [Gradle](https://gradle.org/) takes care of
dependency management, we do not provide a VM image or Docker container.

GDBMeter requires the following software installed:

- [JDK 11](https://www.oracle.com/java/technologies/downloads/)
- [Gradle 8.0.2](https://gradle.org/) which we provide bundled with our source code
- The graph database you want to test ([Neo4J](https://neo4j.com/) and [JanusGraph](https://janusgraph.org/) are both embedded into the JAR for ease of use)
    - We recommend testing [RedisGraph](https://redis.io/docs/stack/graph/) via a Docker container

To download GDBMeter, build it, and run the predicate partitioning on Neo4J, run the following:

```shell
./gradlew clean shadowJar
cd build/libs
java -jar gdbmeter-*.jar -db neo4j -o partition -v
```

It is expected that progress information, similar to the following, is printed:

```
   _____ _____  ____  __  __      _            
  / ____|  __ \|  _ \|  \/  |    | |           
 | |  __| |  | | |_) | \  / | ___| |_ ___ _ __ 
 | | |_ | |  | |  _ <| |\/| |/ _ \ __/ _ \ '__|
 | |__| | |__| | |_) | |  | |  __/ ||  __/ |   
  \_____|_____/|____/|_|  |_|\___|\__\___|_|   
                                               
                                               
Version: 1.0
Selected Database: NEO4J

INFO  c.e.ast.gdbmeter.common.GlobalState - CREATE (ZI:weW {oh:date('0623-08-15'), fgL:duration.inSeconds(date('0352-07-13'),date('00641020')), wRpzDTUl:-0x5662df059a3d14a7}), (k:weW {wRpzDTUl:(-(ZI.wRpzDTUl))/(ZI.wRpzDTUl)}), (m:jBOXf6jc {qg7pf0GJ:duration('P1555136800Y68493577D')}), (MkWtNP:Eb {U:0o31561116576273126202}), (k)-[FgegJc59u3:nqMj {V9F_:k.wRpzDTUl}]->(MkWtNP), (m)-[e0MEC0cb:vyaJ {qy:point({ x: 0.802749, y: 0.527289, z: 0.462671 }), ebllS:exp(abs(0x7c841c43d9ba5b1c)), xOv:+(+(-(toFloat("C/Gs5"))))}]->(ZI), (m)-[Bc31:nqMj {V9F_:-0x1b058b00a6450daf}]->(MkWtNP), (ZI)-[U:vyaJ {qy:point({ x: 0.196766, y: 0.034608 }), ebllS:0.24534499645233154, v5hEURk96A:duration('P379774739Y46746829W511752437DT317248392M684273287S'), xOv:0.232354998588562}]->(m), (ZI)-[ds:nqMj {V9F_:0x39d707e5b4e9a21d}]->(ZI), (ZI)-[CcO:nqMj {V9F_:ZI.wRpzDTUl}]->(MkWtNP), (MkWtNP)-[Gxbk:nqMj {V9F_:m.zGU}]->(m) RETURN DISTINCT CcO , ZI , ds.V9F_ ORDER BY CcO , ZI 
INFO  c.e.ast.gdbmeter.common.GlobalState - CREATE RANGE INDEX G IF NOT EXISTS FOR (n:w) ON (n.pmRMW2M0pp, n.GzUQbgt)
...
```

When running GDBMeter in verbose mode (`-v`) the queries that were executed are printed periodically. While GDBMeter generates syntactically correct statements and queries, not all of them can be
successfully executed by the GDBMS. For example, some functions only allow a limited set of parameters which we cannot determine when generating the queries. As another example, a query can fail when
division-by-zero error occurs.

The shortcut CTRL+C can be used to terminate GDBMeter manually. If GDBMeter does not find any bugs, it executes infinitely. To see all available options one can use the `-h` option.

The following GDBMS are supported when running the Partition testing oracle:

- [RedisGraph](https://redis.io/docs/stack/graph/)
- [Neo4J](https://neo4j.com/)
- [JanusGraph](https://janusgraph.org/)

To test the RedisGraph database, an external server has to be started. To do this we recommend the following docker command which starts the server in the appropriate version:

```shell
docker run -p 6379:6379 -it --rm redis/redis-stack-server:6.2.2-v5
```

## SQLite Database

To view the database's content you can install [SQLite](https://www.sqlite.org/download.html) and use its CLI:

```
sqlite3 bugs.db
sqlite> SELECT COUNT(*) FROM DBMS_BUGS_TRUE_POSITIVES;
```

If the output of the query is `38`, using the database seems to work.

# Detailed Instructions

We recommend performing the following steps to become familiar with the artifact.

## Getting an overview of GDBMeter and the artifact

1. Reading the paper provides more details on the approach and implementation.
2. As of 2022, we are actively extending and improving GDBMeter. Check out the [GitHub repository](https://github.com/gdbmeter/gdbmeter) for the latest version of GDBMeter.
3. Investigate the bugs that we found and summarized in the paper. See Section "Investigating the bugs found" for details.
4. Run GDBMeter with the predicate partition oracle. See Section "Using GDBMeter with predicate partitioning" for details.

## Investigating the bugs found

Next, we recommend investigating the SQLite bug database, which contains all the bug reports found within the scope of this work.

The database contains the following tables:

* `DBMS_BUGS`: This is the main table, and represents all bugs we reported with additional information, such as the GDBMS for which we reported the bug, bug report status, and test case.
* `BUG_TEST_CASES`: This table breaks down the `TEST` column in `DMBS_BUGS` into individual statements.

The views defined on these table aggregate or filter specific information. For example, the `DBMS_BUGS_TRUE_POSITIVES` view excludes the false positives from `DBMS_BUGS`.

In order to understand and replicate our results, we recommend following a top-down approach, that is, find relevant numbers in the paper, execute (some of) the queries below to reproduce the
numbers, and then analyze the query and intermediate outputs to understand how the numbers are computed.

We have adapted this bug database format from [sqlancer/bugs](https://github.com/sqlancer/bugs) with explicit consent of the original authors.

## Numbers Mentioned in the Abstract

New bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE STATUS != "closed (not a bug)";
```

Logic bugs which are also new bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE TYPE = "logic"
  AND STATUS != "closed (not a bug)";
```

Fixed bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE STATUS = "fixed";
```

Confirmed (either fixed or awaiting a fix) bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE STATUS IN ("verified", "fixed");
```

New bugs found in Neo4j and JanusGraph:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE DBMS IN ("Neo4j", "Janus");
```

## Numbers Mentioned in Section "1 Introduction"

New bugs found:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE STATUS != "closed (not a bug)";
```

Fixed bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE STATUS = "fixed";
```

Logic bugs which are also new bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE TYPE = "logic"
  AND STATUS != "closed (not a bug)";
```

New bugs found in Neo4j and JanusGraph:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE DBMS IN ("Neo4j", "Janus");
```

New logic bugs found in Neo4j and JanusGraph:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE DBMS IN ("Neo4j", "Janus")
  AND TYPE = "logic";
```

## Numbers Mentioned in Section "5.1 Effectiveness"

Total reported bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE STATUS != "closed (not a bug)";
```

New bugs found:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE STATUS != "closed (not a bug)";
```

Confirmed (either fixed or awaiting a fix) bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE STATUS IN ("verified", "fixed");
```

Fixed bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE STATUS = "fixed";
```

Duplicate bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE STATUS = "closed (duplicate)";
```

Logic bugs which are also new bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE TYPE = "logic"
  AND STATUS != "closed (not a bug)";
```

Crash bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE TYPE = "crash";
```

Crash bugs which were not found in RedisGraph:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE TYPE = "crash"
  AND DBMS != "RedisGraph";
```

Error bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE TYPE = "error";
```

Hanging bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE TYPE = "hang";
```

Jedis client bugs:

```sql
SELECT COUNT(*)
FROM DBMS_BUGS
WHERE DBMS = "Jedis";
```

## Numbers Mentioned in Table 3

This constructs the table without the totals. In the paper we omit the row where `DBMS` is set to `Jedis`.

```sql
SELECT DBMS,
       sum(case STATUS when "fixed" then 1 else 0 end)                        AS "Fixed",
       sum(case STATUS when "fixed" then 1 when "verified" then 1 else 0 end) AS "Verified",
       sum(case STATUS when "closed (not a bug)" then 1 else 0 end)           AS "Intended",
       sum(case STATUS when "closed (duplicate)" then 1 else 0 end)           AS "Duplicate"
FROM DBMS_BUGS
GROUP BY DBMS
ORDER BY CASE DBMS
             WHEN "Neo4j" THEN 0
             WHEN "RedisGraph" THEN 1
             WHEN "JanusGraph" THEN 2
             WHEN "Jedis" THEN 3
             END
```

## Numbers Mentioned in Table 4

This constructs the table without the totals. In the paper we omit the row where `DBMS` is set to `Jedis`.

```sql
SELECT DBMS,
       sum(case TYPE when "logic" then 1 else 0 end) AS "Logic",
       sum(case TYPE when "crash" then 1 else 0 end) AS "Crash",
       sum(case TYPE when "error" then 1 else 0 end) AS "Error",
       sum(case TYPE when "hang" then 1 else 0 end)  AS "Hang"
FROM DBMS_BUGS
GROUP BY DBMS
ORDER BY CASE DBMS
             WHEN "Neo4j" THEN 0
             WHEN "RedisGraph" THEN 1
             WHEN "JanusGraph" THEN 2
             WHEN "Jedis" THEN 3
             END
```

## Reproducing the bugs from the database

In the database, the `TEST` column in the `DBMS_BUGS` table contains the manually reduced test cases. The `URL_BUGTRACKER` points toward the location where the bug was first reported. Important
additional information about the bug can sometimes be found in the column `COMMENT`.

Consider the bug report with a value of [https://github.com/RedisGraph/RedisGraph/issues/2433](https://github.com/RedisGraph/RedisGraph/issues/2433) in the `URL_BUGTRACKER` column. The test case
described in the issue (and contained in the `TEST` column) is the following:

```
CREATE INDEX FOR (n:L) ON (n.p)
MATCH (n:L) WHERE distance(point({ longitude: 8.076489, latitude: -58.167002 }), n.p) <= -(round(0.7784249186515808)) RETURN n
```

The bug report states that we found the bug in version 6.2.5 of RedisGraph, in commit [02c9cfd](https://github.com/RedisGraph/RedisGraph/commit/02c9cfd66b3281f9bc34914f3522656372ff1b8b) to be precise.
To reproduce the bug we first check out this specific version of RedisGraph and build the database:

```
git clone --recurse-submodules -j8 https://github.com/RedisGraph/RedisGraph.git
cd RedisGraph
git checkout 02c9cfd66b3281f9bc34914f3522656372ff1b8b
make
```

To start the new database server we run the binary `src/redisgraph.so`. We can reproduce the bug by executing the test case in the [redis-cli](https://redis.io/docs/ui/cli/):

```
GRAPH.QUERY graph "CREATE INDEX FOR (n:L) ON (n.p)"
GRAPH.QUERY graph "MATCH (n:L) WHERE distance(point({ longitude: 8.076489, latitude: -58.167002 }), n.p) <= -(round(0.7784249186515808)) RETURN n"
```

As reported in the issue, the second query runs forever.

## Using GDBMeter with predicate partitioning

We recommend experimenting with the predicate partitioning implementations of Neo4J and/or RedisGraph, since for embedded GDBMS, no separate database needs to be set up:

```
java -jar gdbmeter-*.jar -db neo4j -o partition -v
java -jar gdbmeter-*.jar -db redis -o partition -v
```

While it is expected that only status information is printed, it is also possible that bugs are found. In such a case, an exception is thrown, which also indicates the unexpected result. As pointed
out in the `Getting Started` section, GDBMeter by default runs infinitely, if it does not find any bugs, and must be terminated manually.

It logs all statements and queries if the option `-v` is supplied, which allows studying the queries while they are generated. It is also possible to use a Java debugger to step through the code while
it is executed. To do this we recommend using [IntelliJ](https://www.jetbrains.com/idea/) which offers excellent debugging support.

The oracle classes are a good entry point into the code (e.g.,[Neo4JPartitionOracle.java](gdbmeter/src/main/java/ch/ethz/ast/gdbmeter/neo4j/oracle/Neo4JPartitionOracle.java) which implements
the [CypherPartitionOracle.java](gdbmeter/src/main/java/ch/ethz/ast/gdbmeter/cypher/oracle/CypherPartitionOracle.java)). These classes contain the logic of the partition oracle for the specific GDBMS
and are rather compact.

The oracles are used after a fresh database has been created. If you want to investigate the oracles role in the overall testing process, you can investigate one of the provider classes (
e.g., [Neo4JProvider.java](gdbmeter/src/main/java/ch/ethz/ast/gdbmeter/neo4j/Neo4JProvider.java)). In the provider classes, statements are generated to insert, modify and query data of the graphs.

# Claims of the paper supported by the artifact

The artifact provides evidence

- for the bugs that we found
- that the implementation is as described in the paper

The artifact *does not* provide infrastructure to reproduce a specific bug. GDBMeter and the GDBMS evolved during the months in which we tested the GDBMS, so a bug might be reproducible only using a
specific GDBMeter and GDBMS implementation. Since we did not foresee this use case, we only kept track of the GDBMS version in which we found the bug, but not the version of GDBMeter. Even if this
information would be available, multiple bugs could be triggered during one GDBMeter execution, making it necessary to provide a bisection approach. Naive bisection would not work well, since building
a single GDBMS typically takes up to half an hour.
